----------------------------------------------------------------------------------
-- Engineer: Catherine Lagunzad
-- Create Date: 11/19/2017 12:13:43 PM
-- Description: implements a 2-player competitive buzzer system that indicates
-- the first player to answer with an LED and locks out other contestants.
-- It also counts down from 20 or 5 seconds depending on the user input. The
-- buzzer outputs a sound when the timer has reached zero. Note that this module
-- is based on the the buzzers used in the National Science Bowl Competition
-- (only with 2 players)
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity competitive_buzzer_system is
    Port ( player1 : in STD_LOGIC;
           player2 : in STD_LOGIC;
           reset : in STD_LOGIC;
           count_down_20_sec : in STD_LOGIC;
           count_down_5_sec : in STD_LOGIC;
           CLK : in STD_LOGIC;
           speaker : out STD_LOGIC;
           speaker_LED : out STD_LOGIC;
           SEGMENTS : out STD_LOGIC_VECTOR (7 downto 0);
           DISP_EN : out STD_LOGIC_VECTOR (3 downto 0);
           player_LED : out STD_LOGIC_VECTOR (1 downto 0));
end competitive_buzzer_system;

architecture Behavioral of competitive_buzzer_system is

--player lockout
component player_lockout_LED is
    Port ( player1 : in STD_LOGIC;
           player2 : in STD_LOGIC;
           reset : in STD_LOGIC;
           CLK : in STD_LOGIC;
           player_LED : out STD_LOGIC_VECTOR (1 downto 0));
end component;

--buzzer sound
component buzzer_tone is
    Port ( EN : in STD_LOGIC;
           CLK : in STD_LOGIC;
           speaker : out STD_LOGIC;
           speaker_LED : out STD_LOGIC);
end component;

--clock divider into seconds
component clk_div2 is
    Port ( clk : in std_logic;
           sclk : out std_logic);
end component;

--20second or 5second countdown timer
component down_counter_FSM is
    Port ( SEC_CLK : in STD_LOGIC;
           reset : in STD_LOGIC;
           count_down_20_sec : in STD_LOGIC;
           count_down_5_sec : in STD_LOGIC;
           counter_out : out STD_LOGIC_VECTOR(7 downto 0);
           valid_count : out STD_LOGIC;
           buzzer_timeout : out STD_LOGIC);
end component;

--7 segment decoder
component sseg_dec is
    Port ( ALU_VAL : in std_logic_vector(7 downto 0); 
           SIGN : in std_logic;
           VALID : in std_logic;
           CLK : in std_logic;
           DISP_EN : out std_logic_vector(3 downto 0);
           SEGMENTS : out std_logic_vector(7 downto 0));
end component;

--instantiate intermediate signals
signal buzzer_enable, temp_clock, counter_on : STD_LOGIC;
signal countdown_timer_out : STD_LOGIC_VECTOR (7 downto 0);

begin

player_lockout_LED1 : player_lockout_LED
    port map ( player1 => player1,
               player2 => player2,
               reset => reset,
               CLK => CLK,
               player_LED => player_LED);

buzzer_tone1 : buzzer_tone
    port map ( CLK => CLK,
               EN => buzzer_enable,
               speaker => speaker,
               speaker_LED => speaker_LED);
               
clk_div1 : clk_div2
    port map ( clk => CLK,
               sclk => temp_clock);

down_counter_FSM1 : down_counter_FSM
    port map ( SEC_CLK => temp_clock,
               reset => reset,
               count_down_20_sec => count_down_20_sec,
               count_down_5_sec => count_down_5_sec,
               counter_out => countdown_timer_out,
               valid_count => counter_on,
               buzzer_timeout => buzzer_enable);
               
sseg_dec1 : sseg_dec
    port map ( ALU_VAL => countdown_timer_out,
               SIGN => '0',
               VALID => counter_on,
               CLK => CLK,
               DISP_EN => DISP_EN,
               SEGMENTS => SEGMENTS);
end Behavioral;
